" stringliteral.vim
"
" A VIM plugin for creating multi-line string literals
" in C++ programs.
"
" Author: Alcides Viamontes Esquivel
"

if exists("b:did_stringliteral_ftplugin")
  finish
endif
let b:did_stringliteral_ftplugin = 1

function! CppStringLiteralInit()
python << EOF

import vim
import re 
_string_literal_re = re.compile(r"^(\s*)(\"(\\.|[^\"])*\")(\s*)$")

_replace_dict = {
    "\\": "\\\\",
    "\t": "\\t",
    "\"": "\\\""
}

_unreplace_dict = {
}

for (key, val) in _replace_dict.items():
    _unreplace_dict[ val ] = key

def escape_literal( lit ):
    result = "\""
    for c in lit:
        n = _replace_dict.get( c, c ) 
        result += n
    result += "\\n\""
    return result

def unescape_literal( lit ):
    result = ""
    last_char = len( lit ) - 1
    skip = (0, last_char)
    i = 0
    while i < len( lit ): 
        c = lit[i]
        if i in skip:
            i += 1
        else:
            if i < len(lit)-1:
                es = lit[i:i+2]
            else:
                es = None
            if es == '\\n':
                result += '\n'
                i += 2
            else:
                if es in _unreplace_dict:
                    ch = _unreplace_dict[ es ]
                    i+=2
                else:
                    result += c 
                    i+=1
    return result
#vim.command(":echom \"Initted\"\n")
EOF
endfunction

function! CppStringLiteralLiteralize() range
python << EOF
start_line = vim.current.buffer.mark('<')[0]
end_line   = vim.current.buffer.mark('>')[0]
#vim.command(":echom \"{0} - {1} \"\n".format( start_line, end_line ) )
new_content = []
done_something = False
#vim.command(":echom \"arg\"\n")
current_lines = vim.current.buffer[start_line-1:end_line]
for (i,line) in enumerate(current_lines):
    mo = re.match( _string_literal_re, line)
    if mo:
        # Nothing to do
        new_content.append( line )
        #vim.command(":echom \"a\"\n")
    else:
        #vim.command(":echom \"b\"\n")
        #vim.command(":echom \"{0}\"\n".format(len(line)))
        done_something = True
        new_line = escape_literal( line )
        new_content.append( new_line )

#vim.command(":echom \"done\"\n")
if done_something:
    vim.current.buffer[start_line-1:end_line] = new_content
EOF
endfunction

function! CppStringLiteralUnliteralize() range
python << EOF
start_line = vim.current.buffer.mark('<')[0]
end_line   = vim.current.buffer.mark('>')[0]
#vim.command(":echom \"{0} - {1} \"\n".format( start_line, end_line ) )
new_content = []
done_something = False
#vim.command(":echom \"arg\"\n")
current_lines = vim.current.buffer[start_line-1:end_line]
left_over = ""
for (i,line) in enumerate(current_lines):
    mo = re.match( _string_literal_re, line)
    if mo:
        just_literal = mo.group( 2 ) 
        unescaped_result = unescape_literal( just_literal )
        line_list = unescaped_result.split('\n')
        line_list[0] = left_over + line_list[0]
        left_over = line_list[-1]
        new_content.extend( line_list[:-1] )
        #vim.command(":echom \"match\"\n")
        done_something = True
    else:
        # Nothing to do
        new_content.append( line )
        #vim.command(":echom \"nomatch\"\n")
if left_over != "" :
    new_content.append( left_over )
if done_something:
    vim.current.buffer[start_line-1:end_line] = new_content
EOF
endfunction

call CppStringLiteralInit()


